@@ -14,6 +14,7 @@ import posixpath
 import re
 import os
 import codecs
+import traceback
 try:
     from hashlib import sha1 as sha
 except ImportError:
@@ -26,7 +27,7 @@ from sphinx.errors import SphinxError
 from sphinx.util.osutil import ensuredir, ENOENT, EPIPE
 from sphinx.util.compat import Directive
 
-from blockdiag_sphinxhelper import command, diagparser, builder, DiagramDraw
+from blockdiag_sphinxhelper import command, parser, builder, drawer
 from blockdiag_sphinxhelper import collections, FontMap
 from blockdiag_sphinxhelper import blockdiag, BlockdiagDirective
 namedtuple = collections.namedtuple
@@ -40,7 +41,7 @@ class Blockdiag(BlockdiagDirective):
     def run(self):
         try:
             return super(Blockdiag, self).run()
-        except diagparser.ParseException, e:
+        except parser.ParseException, e:
             if self.content:
                 msg = '[%s] ParseError: %s\n%s' % (self.name, e, "\n".join(self.content))
             else:
@@ -57,10 +58,10 @@ def get_image_filename(self, code, format, options, prefix='blockdiag'):
     """
     Get path of output file.
     """
-    if format not in ('PNG', 'PDF', 'SVG'):
+    if format.upper() not in ('PNG', 'PDF', 'SVG'):
         raise BlockdiagError('blockdiag error:\nunknown format: %s\n' % format)
 
-    if format == 'PDF':
+    if format.upper() == 'PDF':
         try:
             import reportlab
         except ImportError:
@@ -125,7 +126,7 @@ def get_anchor(self, refid, fromdocname):
     for docname in self.builder.env.found_docs:
         doctree = self.builder.env.get_doctree(docname)
         for target in doctree.traverse(nodes.Targetable):
-            if target.attributes['refid'] == refid:
+            if target.attributes.get('refid') == refid:
                 targetfile = self.builder.get_relative_uri(fromdocname, docname)
                 return targetfile + "#" + refid
 
@@ -148,17 +149,20 @@ def create_blockdiag(self, code, format, filename, options, prefix):
     draw = None
     fontmap = get_fontmap(self)
     try:
-        tree = diagparser.parse(diagparser.tokenize(code))
+        tree = parser.parse_string(code)
         screen = builder.ScreenNodeBuilder.build(tree)
         for node in screen.traverse_nodes():
             if node.href:
                 node.href = resolve_reference(self, node.href, options)
 
         antialias = self.builder.config.blockdiag_antialias
-        draw = DiagramDraw.DiagramDraw(format, screen, filename,
-                                       fontmap=fontmap, antialias=antialias)
+        draw = drawer.DiagramDraw(format, screen, filename,
+                                  fontmap=fontmap, antialias=antialias)
 
     except Exception, e:
+        if self.builder.config.blockdiag_debug:
+            traceback.print_exc()
+
         raise BlockdiagError('blockdiag error:\n%s\n' % e)
 
     return draw
@@ -189,7 +193,7 @@ def make_imgtag(self, image, relfn, trelfn, outfn,
 
     if clickable_map:
         imgtag_format = '<img src="%s" alt="%s" width="%s" '
-        imgtag_format += 'usemap="#map_1" height="%s" />\n'  # TODO:mapname
+        imgtag_format += 'usemap="#map_%d" height="%%s" />\n' % id(image)
     else:
         imgtag_format = '<img src="%s" alt="%s" width="%s" height="%s" />\n'
 
@@ -203,7 +207,7 @@ def make_imgtag(self, image, relfn, trelfn, outfn,
                    (relfn, alt, image_size[0], image_size[1]))
 
     if clickable_map:
-        result += ('<map name="map_1">')
+        result += ('<map name="map_%d">' % id(image))
         rect_format = '<area shape="rect" coords="%s,%s,%s,%s" href="%s">'
         for m in clickable_map:
             x1 = m[0].x1
@@ -261,7 +265,7 @@ def render_dot_html(self, node, code, options, prefix='blockdiag',
         if alt is None:
             alt = node.get('alt', self.encode(code).strip())
 
-        if format == 'SVG':
+        if format.upper() == 'SVG':
             tagfunc = make_svgtag
         else:
             tagfunc = make_imgtag
@@ -301,7 +305,7 @@ def latex_visit_blockdiag(self, node):
 
 
 def on_doctree_resolved(self, doctree, docname):
-    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex'):
+    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex', 'epub'):
         return
 
     for node in doctree.traverse(blockdiag):
@@ -329,6 +333,7 @@ def setup(app):
     app.add_config_value('blockdiag_fontpath', None, 'html')
     app.add_config_value('blockdiag_fontmap', None, 'html')
     app.add_config_value('blockdiag_antialias', False, 'html')
+    app.add_config_value('blockdiag_debug', False, 'html')
     app.add_config_value('blockdiag_html_image_format', 'PNG', 'html')
     app.add_config_value('blockdiag_tex_image_format', 'PNG', 'html')
     app.connect("doctree-resolved", on_doctree_resolved)
